@@ -1,4 +1,4 @@
-// 
+//
 // Notice Regarding Standards.  AMD does not provide a license or sublicense to
 // any Intellectual Property Rights relating to any standards, including but not
 // limited to any audio and/or video codec technologies such as MPEG-2, MPEG-4;
@@ -6,9 +6,10 @@
 // (collectively, the "Media Technologies"). For clarity, you will pay any
 // royalties due for such third party technologies, which may include the Media
 // Technologies that are owed as a result of AMD providing the Software to you.
-// 
-// MIT license 
-// 
+//
+// MIT license
+//
+//
 // Copyright (c) 2018 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -45,6 +46,10 @@
     #include <codecvt>
 #endif
 
+#if !defined(__APPLE__) && !defined(_WIN32)
+#include <malloc.h>
+#endif
+
 #pragma warning(disable: 4996)
 
 #if defined(__linux) || defined(__APPLE__)
@@ -503,7 +508,7 @@ amf_wstring AMF_STD_CALL amf::amf_from_url_utf8_to_unicode(const amf_string& dat
             Result += data.substr(old_pos);
             break;
         }
-        if(pos - old_pos > 0)
+        if(pos > old_pos)
         {
             Result += data.substr(old_pos, pos - old_pos);
         }
@@ -573,7 +578,7 @@ amf_string AMF_STD_CALL amf::amf_string_format(const char* format, ...)
 //----------------------------------------------------------------------------------------
 amf_wstring AMF_STD_CALL amf::amf_string_formatVA(const wchar_t* format, va_list args)
 {
-#if defined(__linux) || defined(__APPLE__)
+#if (defined(__linux) || defined(__APPLE__)) && (!defined(__ANDROID__))
     //replace %s with %ls
     amf_wstring text(format);
     amf_wstring textReplaced;
@@ -586,6 +591,10 @@ amf_wstring AMF_STD_CALL amf::amf_string_formatVA(const wchar_t* format, va_list
             textReplaced.push_back(L'l');
             textReplaced.push_back(L's');
         }
+        else if(percentFlag && (*i == L'S'))
+        {
+            textReplaced.push_back(L's');
+        }
         else
         {
             textReplaced.push_back(*i);
@@ -679,6 +688,28 @@ void AMF_STD_CALL amf_free(void* ptr)
     free(ptr);
 }
 //----------------------------------------------------------------------------------------
+void* AMF_STD_CALL amf_aligned_alloc(size_t count, size_t alignment)
+{
+#if defined(_WIN32)
+    return _aligned_malloc(count, alignment);
+#elif defined (__APPLE__)
+    void* p = nullptr;
+    posix_memalign(&p, alignment, count);
+    return p;
+#elif defined(__linux)
+    return memalign(alignment, count);
+#endif
+}
+//----------------------------------------------------------------------------------------
+void AMF_STD_CALL amf_aligned_free(void* ptr)
+{
+#if defined(_WIN32)
+    return _aligned_free(ptr);
+#else
+    return free(ptr);
+#endif
+}
+//----------------------------------------------------------------------------------------
 #if defined (__ANDROID__)
 template <typename CHAR_T>
 static bool isOneOf(CHAR_T p_ch, const CHAR_T* p_set)
@@ -1260,7 +1291,7 @@ extern "C"
         amf_wstring low_s2 = s2;
         std::transform(low_s1.begin(), low_s1.end(), low_s1.begin(), ::tolower);
         std::transform(low_s2.begin(), low_s2.end(), low_s2.begin(), ::tolower);
-        
+
         return wcscmp(low_s1.c_str(), low_s2.c_str());
     }
 }
