@@ -1,4 +1,4 @@
-// 
+//
 // Notice Regarding Standards.  AMD does not provide a license or sublicense to
 // any Intellectual Property Rights relating to any standards, including but not
 // limited to any audio and/or video codec technologies such as MPEG-2, MPEG-4;
@@ -6,9 +6,9 @@
 // (collectively, the "Media Technologies"). For clarity, you will pay any
 // royalties due for such third party technologies, which may include the Media
 // Technologies that are owed as a result of AMD providing the Software to you.
-// 
-// MIT license 
-// 
+//
+// MIT license
+//
 //
 // Copyright (c) 2018 Advanced Micro Devices, Inc. All rights reserved.
 //
@@ -69,10 +69,10 @@ class MyPlaybackPipeline : public PlaybackPipeline
      {
          if(m_pVideoPresenter != nullptr)
          {
-             m_pVideoPresenter->CheckForResize();
+             m_pVideoPresenter->ResizeIfNeeded();
          }
      }
-    
+
 };
 
 static MyPlaybackPipeline *s_pPipeline = NULL;
@@ -84,7 +84,7 @@ static MyPlaybackPipeline *s_pPipeline = NULL;
 #define MAX_LOADSTRING 100
 
 // Global Variables:
-static HWND    hToolbar; 
+static HWND    hToolbar;
 TCHAR szTitle[MAX_LOADSTRING];                    // The title bar text
 TCHAR szWindowClass[MAX_LOADSTRING];            // the main window class name
 static  UINT_PTR uiTimerID = 0;
@@ -107,6 +107,7 @@ void                CloseToolbar();
 
 HWND                hMainWindow;
 HWND                hClientWindow;
+HMENU               hMenu;
 
 static INT_PTR CALLBACK ProgressDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);
 
@@ -140,8 +141,9 @@ int APIENTRY _tWinMain(HINSTANCE hInstance,
 int main(int argc, char* argv[])
 {
     int nCmdShow = 0;
+    XInitThreads();
     hInst = amf_handle(XOpenDisplay(NULL));
-    if (hInst == NULL) 
+    if (hInst == NULL)
     {
         printf("Cannot open display\n");
         return -1;
@@ -162,7 +164,7 @@ int main(int argc, char* argv[])
     }
 
     {
-        amf::AMFTraceSetGlobalLevel(AMF_TRACE_INFO); 
+        amf::AMFTraceSetGlobalLevel(AMF_TRACE_INFO);
         amf::AMFTraceEnableWriter(AMF_TRACE_WRITER_DEBUG_OUTPUT, true);
         amf::AMFTraceSetWriterLevel(AMF_TRACE_WRITER_DEBUG_OUTPUT, AMF_TRACE_INFO);
 
@@ -175,7 +177,7 @@ int main(int argc, char* argv[])
         if (parseCmdLineParameters(s_pPipeline))
 #else
         if (parseCmdLineParameters(s_pPipeline, argc, argv))
-#endif        
+#endif
         {
 
             if (!InitInstance(nCmdShow))
@@ -199,6 +201,8 @@ int main(int argc, char* argv[])
                 s_pPipeline->Play();
                 UpdateMenuItems();
             }
+            ret = RunMessageLoop();
+            s_pPipeline->Stop();
         }
         else
         {
@@ -206,8 +210,6 @@ int main(int argc, char* argv[])
         }
         //------------------------------------------------------------------------------------------------------------
 
-        ret = RunMessageLoop();
-        s_pPipeline->Stop();
 
         CloseInstance();
     }
@@ -258,7 +260,6 @@ ATOM MyRegisterClass()
     WNDCLASSEX wcex;
 
     wcex.cbSize = sizeof(WNDCLASSEX);
-
     wcex.style              = CS_HREDRAW | CS_VREDRAW;
     wcex.lpfnWndProc        = WndProc;
     wcex.cbClsExtra         = 0;
@@ -309,15 +310,42 @@ bool InitInstance(int nCmdShow)
 
     ShowWindow(hMainWindow, nCmdShow);
     UpdateWindow(hMainWindow);
+    hMenu = ::GetMenu(hMainWindow);
     UpdateMenuItems();
 
     return true;
 }
 
+void UpdateWindowStyle()
+{
+    amf::AMF_MEMORY_TYPE presenterType = amf::AMF_MEMORY_DX11;
+
+    amf_int64 engineInt = amf::AMF_MEMORY_UNKNOWN;
+    if (s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_PRESENTER, engineInt) != AMF_OK || engineInt == amf::AMF_MEMORY_UNKNOWN)
+    {
+        return;
+    }
+
+    presenterType = (amf::AMF_MEMORY_TYPE)engineInt;
+
+    const LONG classStyle = presenterType == amf::AMF_MEMORY_VULKAN ? 0 : (CS_HREDRAW | CS_VREDRAW);
+
+    // Don't redraw on vertical/horizontal resizing
+    // This messes with vulkan swapchain causing flickering
+    SetClassLongPtr(hMainWindow, GCL_STYLE, classStyle);
+}
+
 void UpdateMenuItems()
 {
-    PipelineState currState = s_pPipeline->GetState();
-    HMENU         hMenu     = ::GetMenu(hMainWindow);
+    const PipelineState currState = s_pPipeline->GetState();
+
+    bool bFullScreen = false;
+    s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_FULLSCREEN, bFullScreen);
+
+    // Hide the menu on fullscreen. Some presenters like DX11 do this automatically
+    // but some may not like DX12. The problem is that the menu is seen as a "seperate"
+    // window so interacting with it causes DXGI to automatically undo fullscreen
+    ::SetMenu(hMainWindow, bFullScreen && currState == PipelineStateRunning ? nullptr : hMenu);
 
     amf::AMF_MEMORY_TYPE    presenterType = amf::AMF_MEMORY_DX11;
     {
@@ -366,10 +394,17 @@ void UpdateMenuItems()
 	EnableMenuItem(hMenu,ID_HQSCALER_FSR,      MF_BYCOMMAND| ( ((currState == PipelineStateRunning) && isHQScalerOFF) ? MF_DISABLED : MF_ENABLED));
 	EnableMenuItem(hMenu,ID_HQSCALER_OFF,      MF_BYCOMMAND| ( currState == PipelineStateRunning                      ? MF_DISABLED : MF_ENABLED));
 
- 
-    bool bFullScreen = false;
-    s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_FULLSCREEN, bFullScreen);
     CheckMenuItem(hMenu, ID_OPTIONS_FULLSCREEN, MF_BYCOMMAND | (bFullScreen ? MF_CHECKED : MF_UNCHECKED));
+
+
+    bool bExclusiveFullscreen = false;
+    s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_EXCLUSIVE_FULLSCREEN, bExclusiveFullscreen);
+    CheckMenuItem(hMenu, ID_OPTIONS_EXCLUSIVEFULLSCREEN, MF_BYCOMMAND | (bExclusiveFullscreen ? MF_CHECKED : MF_UNCHECKED));
+
+    amf_int64 frcModeType = FRC_OFF;
+    s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_FRC_MODE, frcModeType);
+    CheckMenuItem(hMenu, ID_OPTIONS_FRC, MF_BYCOMMAND | (frcModeType != FRC_OFF ? MF_CHECKED : MF_UNCHECKED));
+    EnableMenuItem(hMenu, ID_OPTIONS_FRC, MF_BYCOMMAND | (currState == PipelineStateRunning ? MF_DISABLED : MF_ENABLED));
 }
 
 //
@@ -404,9 +439,12 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
             break;
         case ID_FILE_OPEN:
             FileOpen(hWnd);
+            s_pPipeline->CheckForResize();
+            UpdateWindowStyle();
             break;
         case ID_FILE_OPENNETWORKSTREAM:
             StreamOpen(hWnd);
+            UpdateWindowStyle();
             break;
         case ID_PLAYBACK_PLAY:
             if(s_pPipeline->GetState() == PipelineStateEof || s_pPipeline->GetState() == PipelineStateNotReady)
@@ -415,12 +453,14 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
                 {
                     s_pPipeline->Play();
                     UpdateMenuItems();
+                    UpdateWindowStyle();
                 }
             }else if(s_pPipeline->GetState() == PipelineStateRunning)
             {
                 s_pPipeline->Play();
                 UpdateMenuItems();
             }
+            s_pPipeline->CheckForResize();
 
             break;
         case ID_PLAYBACK_PAUSE:
@@ -476,12 +516,30 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
             bool bFullScreen = false;
             s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_FULLSCREEN, bFullScreen);
             s_pPipeline->SetParam(PlaybackPipeline::PARAM_NAME_FULLSCREEN, !bFullScreen);
+            s_pPipeline->CheckForResize();
+            UpdateMenuItems();
+            break;
+        }
+        case ID_OPTIONS_EXCLUSIVEFULLSCREEN:
+        {
+            bool bExclusiveFullScreen = false;
+            s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_EXCLUSIVE_FULLSCREEN, bExclusiveFullScreen);
+            s_pPipeline->SetParam(PlaybackPipeline::PARAM_NAME_EXCLUSIVE_FULLSCREEN, !bExclusiveFullScreen);
+            s_pPipeline->CheckForResize();
             UpdateMenuItems();
             break;
         }
         case ID_TOOLBAR:
             ToggleToolbar(hWnd);
             break;
+        case ID_OPTIONS_FRC:
+        {
+            amf_int64 frcModeType = FRC_OFF;
+            s_pPipeline->GetParam(PlaybackPipeline::PARAM_NAME_FRC_MODE, frcModeType);
+            s_pPipeline->SetParam(PlaybackPipeline::PARAM_NAME_FRC_MODE, frcModeType != FRC_OFF ? FRC_OFF : FRC_x2_PRESENT);
+            UpdateMenuItems();
+            break;
+        }
         default:
             return DefWindowProc(hWnd, message, wParam, lParam);
         }
@@ -529,11 +587,11 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 }
 
 
-void FileOpen(HWND hwnd) 
+void FileOpen(HWND hwnd)
 {
     OPENFILENAME ofn;       // common dialog box structure
     WCHAR szFile[260];       // buffer for file name
-     
+
     // Initialize OPENFILENAME
     ZeroMemory(&ofn, sizeof(ofn));
     ofn.lStructSize = sizeof(ofn);
@@ -541,7 +599,7 @@ void FileOpen(HWND hwnd)
     ofn.lpstrFile = szFile;
     ofn.lpstrFile[0] = '\0';
     ofn.nMaxFile = _countof(szFile);
-    ofn.lpstrFilter = L"Videos\0*.WMV;*.WMA;*.AVI;*.ASF;*.FLV;*.BFI;*.CAF;*.GXF;*.IFF;*.RL2;*.MP4;*.3GP;*.QTFF;*.MKV;*.MK3D;*.MKA;*.MKS;*.MPG;*.MPEG;*.PS;*.TS;*.MXF;*.OGV;*.OGA;*.OGX;*.OGG;*.SPX;*.FLV;*.F4V;*.F4P;*.F4A;*.F4B;*.JSV;*.h264;*.264;*.vc1;*.mov;*.mvc;*.m1v;*.m2v;*.m2ts;*.vpk;*.yuv;*.rgb;*.nv12;*.h265;*.265;*.ivf\0All\0*.*\0";
+    ofn.lpstrFilter = L"Videos\0*.WMV;*.WMA;*.AVI;*.ASF;*.FLV;*.BFI;*.CAF;*.GXF;*.IFF;*.RL2;*.MP4;*.3GP;*.QTFF;*.MKV;*.MK3D;*.MKA;*.MKS;*.WEBM;*.MPG;*.MPEG;*.PS;*.TS;*.MXF;*.OGV;*.OGA;*.OGX;*.OGG;*.SPX;*.FLV;*.F4V;*.F4P;*.F4A;*.F4B;*.JSV;*.h264;*.264;*.vc1;*.mov;*.mvc;*.m1v;*.m2v;*.m2ts;*.vpk;*.yuv;*.rgb;*.nv12;*.h265;*.265;*.ivf\0All\0*.*\0";
 //    ofn.lpstrFilter = L"Video streams\0*.h264;*.264;*.h265;*.265;*.hevc;\0All\0*.*\0";
     ofn.nFilterIndex = 1;
     ofn.lpstrFileTitle = NULL;
@@ -585,7 +643,7 @@ HWND CreateClientWindow(HWND hWndParent)
 {
     WNDCLASSEX wcex;
 
-    static wchar_t *ChildClassName = L"Client";
+    static const wchar_t *ChildClassName = L"Client";
     wcex.cbSize = sizeof(WNDCLASSEX);
 
     wcex.style            = CS_HREDRAW | CS_VREDRAW;
@@ -608,7 +666,7 @@ HWND CreateClientWindow(HWND hWndParent)
     }
 
     HWND hClientWnd = CreateWindowEx( 0, ChildClassName, NULL, WS_CHILD | WS_VISIBLE,
-        0, 0, 100, 100, 
+        0, 0, 100, 100,
         hWndParent, (HMENU) (int) (1) , (HINSTANCE)hInst, NULL);
     return hClientWnd;
 }
@@ -656,11 +714,11 @@ static INT_PTR CALLBACK ToolbarDlgProc(HWND hwnd, UINT Message, WPARAM wParam, L
     case WM_HSCROLL:
     {
         if(LOWORD(wParam) == TB_THUMBTRACK)
-        { 
+        {
             bTrackingStarted = true;
         }
         if(LOWORD(wParam) == TB_ENDTRACK)
-        { 
+        {
             amf_int32 pos = (amf_int32)SendMessage(hSlider, TBM_GETPOS , 0, 0);
             s_pPipeline->Seek(static_cast<amf_pts>(s_pPipeline->GetProgressSize() * double(pos) / double(MAX_SLIDER_VALUE)));
             bTrackingStarted = false;
@@ -672,7 +730,7 @@ static INT_PTR CALLBACK ToolbarDlgProc(HWND hwnd, UINT Message, WPARAM wParam, L
         { // update scroll
             double duration =  s_pPipeline->GetProgressSize();
             if(duration != 0)
-            { 
+            {
                 amf_int32 pos = amf_int32(s_pPipeline->GetProgressPosition() * double(MAX_SLIDER_VALUE) / duration);
                 SendMessage(hSlider, TBM_SETPOS , TRUE, pos);
             }
@@ -691,13 +749,13 @@ void ToggleToolbar(HWND hwnd)
         CloseToolbar();
     }
     else
-    { 
+    {
         // modeless dialog
         hToolbar = CreateDialog((HINSTANCE)hInst, MAKEINTRESOURCE(IDD_TOOLBAR_DLG),  hwnd,  (DLGPROC) ToolbarDlgProc );
-        uiTimerToolbarID = ::SetTimer(hToolbar, 10001, 200, NULL); // timer for slider update 
-        ShowWindow(hToolbar, SW_SHOW); 
+        uiTimerToolbarID = ::SetTimer(hToolbar, 10001, 200, NULL); // timer for slider update
+        ShowWindow(hToolbar, SW_SHOW);
     }
-         
+
 }
 //-------------------------------------------------------------------------------------------------
 void CloseToolbar()
@@ -753,7 +811,7 @@ INT_PTR CALLBACK    OpenStreamProc(HWND hDlg, UINT message, WPARAM wParam, LPARA
             s_pPipeline->SetParamAsString(PlaybackPipelineBase::PARAM_NAME_URL_VIDEO, buf);
             GetWindowText(GetDlgItem(hDlg, IDC_EDIT_URL_AUDIO), buf, amf_countof(buf));
             s_pPipeline->SetParamAsString(PlaybackPipelineBase::PARAM_NAME_URL_AUDIO, buf);
-            
+
             bool bListen = IsDlgButtonChecked(hDlg, IDC_CHECK_LISTEN) == BST_CHECKED;
             s_pPipeline->SetParam(PlaybackPipelineBase::PARAM_NAME_LISTEN_FOR_CONNECTION, bListen);
 
@@ -856,17 +914,17 @@ bool                InitInstance(int)
     XWindowAttributes getWinAttr;
     XGetWindowAttributes(dpy, parentWnd, &getWinAttr);
 
-    hMainWindow = XCreateSimpleWindow(dpy, 
-        parentWnd, 
+    hMainWindow = XCreateSimpleWindow(dpy,
+        parentWnd,
 //        10, 10, 800, 600,
-        0, 0, getWinAttr.width, getWinAttr.height, 
+        0, 0, getWinAttr.width, getWinAttr.height,
         1, BlackPixel(dpy, screen_num), WhitePixel(dpy, screen_num));
     XSelectInput(dpy, hMainWindow, ExposureMask | KeyPressMask | StructureNotifyMask);
     XMapWindow(dpy, hMainWindow);
     XStoreName(dpy, hMainWindow, "PlaybackHW");
- 
-    WM_DELETE_WINDOW = XInternAtom(dpy, "WM_DELETE_WINDOW", False); 
-    XSetWMProtocols(dpy, hMainWindow, &WM_DELETE_WINDOW, 1); 
+
+    WM_DELETE_WINDOW = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
+    XSetWMProtocols(dpy, hMainWindow, &WM_DELETE_WINDOW, 1);
     hClientWindow = hMainWindow; // for now
 
     return true;
@@ -877,30 +935,42 @@ int                RunMessageLoop()
     Display* dpy = (Display*)hInst;
     XEvent e;
     bool bRun = true;
-    while (bRun) 
+    while (bRun)
     {
-        XNextEvent(dpy, &e);
-        switch(e.type)
+        // iterate until no events are found
+        while (bRun)
         {
-        case Expose:
-            break;
-        case KeyPress:
-            break;
-        case ConfigureNotify:
+            XLockDisplay(dpy);
+            if (XPending(dpy) == 0)
             {
-                XConfigureEvent xce = e.xconfigure;
-                s_pPipeline->CheckForResize();
+                XUnlockDisplay(dpy);
+                break;
             }
-            break;
-        case ClientMessage:
-            if((static_cast<unsigned int>(e.xclient.data.l[0]) == WM_DELETE_WINDOW))
+            XNextEvent(dpy, &e);
+            XUnlockDisplay(dpy);
+            switch(e.type)
             {
-                bRun = false;
+            case Expose:
+                break;
+            case KeyPress:
+                break;
+            case ConfigureNotify:
+                {
+                    XConfigureEvent xce = e.xconfigure;
+                    s_pPipeline->CheckForResize();
+                }
+                break;
+            case ClientMessage:
+                if((static_cast<unsigned int>(e.xclient.data.l[0]) == WM_DELETE_WINDOW))
+                {
+                    bRun = false;
+                }
+                break;
             }
-            break;
         }
-    }    
-    return 0;    
+        amf_sleep(10);
+    }
+    return 0;
 }
 
 void CloseInstance()
