@@ -53,6 +53,22 @@ static AMF_RESULT ParamConverterUsageAV1(const std::wstring& value, amf::AMFVari
     {
         paramValue = AMF_VIDEO_ENCODER_AV1_USAGE_LOW_LATENCY;
     }
+    else if(uppValue == L"ULTRALOWLATENCY"|| uppValue == L"2")
+    {
+        paramValue = AMF_VIDEO_ENCODER_AV1_USAGE_ULTRA_LOW_LATENCY;
+    }
+    else if(uppValue == L"WEBCAM"|| uppValue == L"3")
+    {
+        paramValue = AMF_VIDEO_ENCODER_AV1_USAGE_WEBCAM;
+    }
+    else if (uppValue == L"HIGHQUALITY" || uppValue == L"HQ" || uppValue == L"4")
+    {
+        paramValue = AMF_VIDEO_ENCODER_AV1_USAGE_HIGH_QUALITY;
+    }
+    else if (uppValue == L"LOWLATENCYHIGHQUALITY" || uppValue == L"LLHQ" || uppValue == L"5")
+    {
+        paramValue = AMF_VIDEO_ENCODER_AV1_USAGE_LOW_LATENCY_HIGH_QUALITY;
+    }
     else {
         LOG_ERROR(L"AMF_VIDEO_ENCODER_AV1_USAGE_ENUM hasn't \"" << value << L"\" value.");
         return AMF_INVALID_ARG;
@@ -78,6 +94,10 @@ static AMF_RESULT ParamConverterAlignmentModeAV1(const std::wstring& value, amf:
     {
         paramValue = AMF_VIDEO_ENCODER_AV1_ALIGNMENT_MODE_NO_RESTRICTIONS;
     }
+    else if (uppValue == L"8X2ONLY" || uppValue == L"4")
+    {
+        paramValue = AMF_VIDEO_ENCODER_AV1_ALIGNMENT_MODE_8X2_ONLY;
+    }
     else {
         LOG_ERROR(L"AMF_VIDEO_ENCODER_AV1_ALIGNMENT_MODE_ENUM hasn't \"" << value << L"\" value.");
         return AMF_INVALID_ARG;
@@ -216,6 +236,41 @@ static AMF_RESULT ParamConverterProfileLevelAV1(const std::wstring& value, amf::
     return AMF_OK;
 }
 
+static AMF_RESULT ParamConverterLTRMode(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_VIDEO_ENCODER_AV1_LTR_MODE_ENUM paramValue;
+    std::wstring uppValue = toUpper(value);
+    if (uppValue == L"RESET_UNUSED" || uppValue == L"0")
+    {
+        paramValue = AMF_VIDEO_ENCODER_AV1_LTR_MODE_RESET_UNUSED;
+    }
+    else if (uppValue == L"KEEP_UNUSED" || uppValue == L"1") {
+        paramValue = AMF_VIDEO_ENCODER_AV1_LTR_MODE_KEEP_UNUSED;
+    }
+    else {
+        LOG_ERROR(L"AMF_VIDEO_ENCODER_AV1_LTR_MODE_ENUM hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
+
+static AMF_RESULT ParamConverterOutputModeAV1(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_VIDEO_ENCODER_AV1_OUTPUT_MODE_ENUM paramValue;
+    std::wstring uppValue = toUpper(value);
+    if(uppValue == L"FRAME" || uppValue == L"0")
+    {
+        paramValue =  AMF_VIDEO_ENCODER_AV1_OUTPUT_MODE_FRAME;
+    } else if(uppValue == L"TILE" || uppValue == L"1") {
+        paramValue =  AMF_VIDEO_ENCODER_AV1_OUTPUT_MODE_TILE;
+    } else {
+        LOG_ERROR(L"AMF_VIDEO_ENCODER_AV1_OUTPUT_MODE_ENUM hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
 static AMF_RESULT ParamConverterCDEFMode(const std::wstring& value, amf::AMFVariant& valueOut)
 {
     AMF_VIDEO_ENCODER_AV1_CDEF_MODE_ENUM paramValue;
@@ -458,6 +513,7 @@ AMF_RESULT RegisterEncoderParamsAV1(ParametersStorage* pParams)
 
     // ------------- Encoder params static---------------
     // Encoder Engine Settings
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_INPUT_QUEUE_SIZE, ParamEncoderStatic, L"AV1 Input Queue size 0, 1 etc, default = 16", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_ENCODING_LATENCY_MODE, ParamEncoderStatic, L"Enables low latency mode ( NONE, PWRSAVING, REALTIME, LOWLATENCY, default =  false)", ParamConverterLatencyModeAV1);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_ENCODER_INSTANCE_INDEX, ParamEncoderStatic, L" Index of VCN instance 0, 1 etc, default = 0", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_QUERY_TIMEOUT, ParamEncoderStatic, L" QueryOutput timeout in ms , default = 0", ParamConverterInt64);
@@ -466,7 +522,7 @@ AMF_RESULT RegisterEncoderParamsAV1(ParametersStorage* pParams)
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_PROFILE, ParamEncoderStatic, L"AV1 profile (Main, default = Main)", ParamConverterProfileAV1);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_TILES_PER_FRAME, ParamEncoderStatic, L"Number of tiles Per Frame. This is treated as suggestion (integer, default = 1)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_LEVEL, ParamEncoderStatic, L"AV1 profile level (float or integer, default = based on HW", ParamConverterProfileLevelAV1);
-    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_QUALITY_PRESET, ParamEncoderStatic, L"Quality Preset (BALANCED, SPEED, QUALITY default = depends on USAGE)", ParamConverterQualityAV1);
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_QUALITY_PRESET, ParamEncoderStatic, L"Quality Preset (BALANCED, SPEED, QUALITY, HIGHQUALITY default = depends on USAGE)", ParamConverterQualityAV1);
     // Codec Configuration
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_SCREEN_CONTENT_TOOLS, ParamEncoderStatic, L"Allow enabling screen content tools (true, false default = depends on USAGE)", ParamConverterBoolean);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_ORDER_HINT, ParamEncoderStatic, L"Code order hint (true, false default =  depends on USAGE)", ParamConverterBoolean);
@@ -474,6 +530,8 @@ AMF_RESULT RegisterEncoderParamsAV1(ParametersStorage* pParams)
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_TILE_GROUP_OBU, ParamEncoderStatic, L"Code FrameHeaderObu + TileGroupObu and each TileGroupObu contains one tile (true, false default =  depends on USAGE)", ParamConverterBoolean);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_CDEF_MODE, ParamEncoderStatic, L"Cdef mode (ENABLE, DISABLE default = depends on USAGE)", ParamConverterCDEFMode);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_ERROR_RESILIENT_MODE, ParamEncoderStatic, L"Enable error resilient mode (true, false default =  depends on USAGE)", ParamConverterBoolean);
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_LTR_MODE, ParamEncoderStatic, L"LTR Mode (RESET_UNUSED = 0, KEEP_UNUSED = 1, default = RESET_UNUSED)", ParamConverterLTRMode);
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_OUTPUT_MODE, ParamEncoderStatic, L"Output Mode (FRAME, TILE, default = FRAME)", ParamConverterOutputModeAV1);
     // Rate Control and Quality Enhancement
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_RATE_CONTROL_METHOD, ParamEncoderStatic, L"Rate Control Method (CQP, CBR, VBR, VBR_LAT default = depends on USAGE)", ParamConverterRateControlAV1);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_INITIAL_VBV_BUFFER_FULLNESS, ParamEncoderStatic, L"Initial VBV Buffer Fullness (integer, 0=0% 64=100% ,default = depends on USAGE)", ParamConverterInt64);
@@ -487,12 +545,16 @@ AMF_RESULT RegisterEncoderParamsAV1(ParametersStorage* pParams)
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MAX_NUM_TEMPORAL_LAYERS, ParamEncoderStatic, L" Max Of LTR frames (integer, default = depends on USAGE)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MAX_LTR_FRAMES, ParamEncoderStatic, L" Max Of LTR frames (integer, default = depends on USAGE)", ParamConverterInt64);
 
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MAX_CONSECUTIVE_BPICTURES, ParamEncoderStatic, L"Max Number of Consecutive B frame (integer 0-127, default = depends on USAGE)", ParamConverterInt64);
+
     // AAA properties
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_ENABLE_SMART_ACCESS_VIDEO, ParamEncoderStatic, L"Enable encoder smart access video feature (bool, default = false)", ParamConverterBoolean);
 
     // AV1 Alignment mode
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_ALIGNMENT_MODE, ParamEncoderStatic, L"AV1 Alignment mode. (64X16ONLY=1, 64X16A1080P=2, NORESTRICTIONS=3, default = 1)", ParamConverterAlignmentModeAV1);
 
+    // Split frame encode flag
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MULTI_HW_INSTANCE_ENCODE, ParamEncoderStatic, L"Enable AV1 encoder split frame encode feature (bool, default = true)", ParamConverterBoolean);
     // ------------- Encoder params dynamic ---------------
     // Codec Configuration
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_PALETTE_MODE, ParamEncoderDynamic, L"Enable palette mode (true, false default =  false)", ParamConverterBoolean);
@@ -512,15 +574,22 @@ AMF_RESULT RegisterEncoderParamsAV1(ParametersStorage* pParams)
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MAX_Q_INDEX_INTRA, ParamEncoderDynamic, L"Max QIndex for intra frame (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MIN_Q_INDEX_INTER, ParamEncoderDynamic, L"Min QIndex for inter frame (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MAX_Q_INDEX_INTER, ParamEncoderDynamic, L"Max QIndex for inter frame (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MIN_Q_INDEX_INTER_B, ParamEncoderDynamic, L"Min QIndex for B-frame (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_MAX_Q_INDEX_INTER_B, ParamEncoderDynamic, L"Max QIndex for  B-frame (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_Q_INDEX_INTRA, ParamEncoderDynamic, L"intra-frame QIndex (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_Q_INDEX_INTER, ParamEncoderDynamic, L"inter-frame QIndex (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_Q_INDEX_INTER_B, ParamEncoderDynamic, L"B-frame QIndex (integer 0-255, default = depends on USAGE)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_RATE_CONTROL_SKIP_FRAME, ParamEncoderDynamic, L"Rate Control Based Frame Skip (true, false default =  depends on USAGE)", ParamConverterBoolean);
     // Picture Management Configuration
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_GOP_SIZE, ParamEncoderDynamic, L"GOP Size (in frames, depends on USAGE)", ParamConverterInt64);
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_INTRA_PERIOD, ParamEncoderDynamic, L"The distance between two intra frames (in frames, default = 0)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_HEADER_INSERTION_MODE, ParamEncoderDynamic, L"Insertion mode (none, gop, idr default = depends on USAGE)", ParamConverterInsertionModeAV1);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_SWITCH_FRAME_INSERTION_MODE, ParamEncoderDynamic, L"Switch frame insertin mode (none, fixed default = depends on USAGE)", ParamConverterSwitchInsertionModeAV1);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_SWITCH_FRAME_INTERVAL, ParamEncoderDynamic, L"The interval between two inserted switch frames (integer, default = depends on USAGE)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_NUM_TEMPORAL_LAYERS, ParamEncoderDynamic, L"Number of temporal layers (integer, default = depends on USAGE)", ParamConverterInt64);
+
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_B_PIC_PATTERN, ParamEncoderDynamic, L"Number of Consecutive B frame (integer 0-127, default = depends on USAGE)", ParamConverterInt64);
+
     //pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_TEMPORAL_LAYER_SELECT, ParamEncoderDynamic, L"Select temporal layer to apply parameter changes and queries (integer, default = 0)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_INTRA_REFRESH_MODE, ParamEncoderDynamic, L"Intra Refresh mode (disable, GOP aligned, continuous, default = disable)", ParamConverterIntraRefreshAV1);
     pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_INTRAREFRESH_STRIPES, ParamEncoderDynamic, L"Frame numbers for an Intra Refresh cycle (integer, default = 0)", ParamConverterInt64);
@@ -553,6 +622,8 @@ AMF_RESULT RegisterEncoderParamsAV1(ParametersStorage* pParams)
     pParams->SetParamDescription(AMF_PA_LOOKAHEAD_BUFFER_DEPTH, ParamEncoderDynamic, L"PA Buffer size (integer 0 - MAX_LOOKAHEAD_DEPTH, default = 0)", ParamConverterInt64);
     pParams->SetParamDescription(AMF_PA_TAQ_MODE, ParamEncoderDynamic, L"TAQ Mode (NONE, 1, 2, default = NONE)", ParamConverterTAQModeAV1);
 
+    pParams->SetParamDescription(AMF_VIDEO_ENCODER_AV1_ADAPTIVE_MINIGOP, ParamEncoderStatic, L"Adaptive MiniGOP Enable default = false)", ParamConverterBoolean);
+
 
     return AMF_OK;
-}
\ No newline at end of file
+}
