@@ -1,4 +1,4 @@
-// 
+//
 // Notice Regarding Standards.  AMD does not provide a license or sublicense to
 // any Intellectual Property Rights relating to any standards, including but not
 // limited to any audio and/or video codec technologies such as MPEG-2, MPEG-4;
@@ -6,9 +6,9 @@
 // (collectively, the "Media Technologies"). For clarity, you will pay any
 // royalties due for such third party technologies, which may include the Media
 // Technologies that are owed as a result of AMD providing the Software to you.
-// 
-// MIT license 
-// 
+//
+// MIT license
+//
 // Copyright (c) 2018 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -41,6 +41,7 @@
 #include "public/include/components/VideoEncoderAV1.h"
 #include "public/include/components/VideoConverter.h"
 #include "public/include/components/HQScaler.h"
+#include "public/include/components/FRC.h"
 
 
 std::wstring AddIndexToPath(const std::wstring& path, amf_int32 index)
@@ -146,7 +147,7 @@ AMF_RESULT ParametersStorage::SetParamAsString(const std::wstring& name, const s
     if(found->second.m_Converter != NULL)
     {
         amf::AMFVariant valueConverted;
-        
+
         AMF_RESULT res = found->second.m_Converter(value, valueConverted);
         if(res != AMF_OK)
         {
@@ -219,7 +220,18 @@ std::wstring  ParametersStorage::GetParamUsage(ParamType type)
     return options;
 }
 
-
+AMF_RESULT  ParametersStorage::CopyTo(ParametersStorage* params)
+{
+    amf_size   count = GetParamCount();
+    for (amf_size i = 0; i < count; i++)
+    {
+        std::wstring name;
+        amf::AMFVariant value;
+        GetParamAt(i, name, &value);
+        params->SetParam(name.c_str(), value);
+    }
+    return AMF_OK;
+}
 
 AMF_RESULT ParamConverterVideoPresenter(const std::wstring& value, amf::AMFVariant& valueOut)
 {
@@ -241,7 +253,7 @@ AMF_RESULT ParamConverterVideoPresenter(const std::wstring& value, amf::AMFVaria
     if(uppValue == L"OPENGL"  || uppValue == L"5")
     {
         paramValue = amf::AMF_MEMORY_OPENGL;
-    }else 
+    }else
     if(uppValue == L"VULKAN"  || uppValue == L"10")
     {
         paramValue = amf::AMF_MEMORY_VULKAN;
@@ -290,40 +302,92 @@ AMF_RESULT ParamConverterFormat(const std::wstring& value, amf::AMFVariant& valu
 {
     amf::AMF_SURFACE_FORMAT paramValue = amf::AMF_SURFACE_UNKNOWN;
     std::wstring uppValue = toUpper(value);
-    if(uppValue == L"NV12" || uppValue == L"1")
+    if (uppValue == L"NV12" || uppValue == L"1")
+    {
+        paramValue = amf::AMF_SURFACE_NV12;
+    }
+    else if (uppValue == L"YV12" || uppValue == L"2")
+    {
+        paramValue = amf::AMF_SURFACE_YV12;
+    }
+    else if (uppValue == L"BGRA" || uppValue == L"3")
+    {
+        paramValue = amf::AMF_SURFACE_BGRA;
+    }
+    else if (uppValue == L"ARGB" || uppValue == L"4")
+    {
+        paramValue = amf::AMF_SURFACE_ARGB;
+    }
+    else if (uppValue == L"RGBA" || uppValue == L"5")
+    {
+        paramValue = amf::AMF_SURFACE_RGBA;
+    }
+    else if (uppValue == L"GRAY8" || uppValue == L"6")
+    {
+        paramValue = amf::AMF_SURFACE_GRAY8;
+    }
+    else if (uppValue == L"YUV420P" || uppValue == L"420P" || uppValue == L"7")
+    {
+        paramValue = amf::AMF_SURFACE_YUV420P;
+    }
+    else if (uppValue == L"U8V8" || uppValue == L"8")
+    {
+        paramValue = amf::AMF_SURFACE_U8V8;
+    }
+    else if (uppValue == L"YUY2" || uppValue == L"9")
+    {
+        paramValue = amf::AMF_SURFACE_YUY2;
+    }
+    else if (uppValue == L"P010" || uppValue == L"10")
+    {
+        paramValue = amf::AMF_SURFACE_P010;
+    }
+    else if (uppValue == L"RGBAF16" || uppValue == L"RGBA_F16" || uppValue == L"11")
+    {
+        paramValue = amf::AMF_SURFACE_RGBA_F16;
+    }
+    else if (uppValue == L"UYVY" || uppValue == L"12")
+    {
+        paramValue = amf::AMF_SURFACE_UYVY;
+    }
+    else if (uppValue == L"R10G10B10A2" || uppValue == L"13")
     {
-        paramValue =  amf::AMF_SURFACE_NV12;
-    } else if(uppValue == L"YV12" || uppValue == L"2") {
-        paramValue =  amf::AMF_SURFACE_YV12;
-    } else if(uppValue == L"BGRA" || uppValue == L"3") {
-        paramValue =  amf::AMF_SURFACE_BGRA;
-    } else if(uppValue == L"ARGB" || uppValue == L"4") {
-        paramValue =  amf::AMF_SURFACE_ARGB;
-    } else if(uppValue == L"RGBA" || uppValue == L"5") {
-        paramValue =  amf::AMF_SURFACE_RGBA;
-    } else if(uppValue == L"GRAY8" || uppValue == L"6") {
-        paramValue =  amf::AMF_SURFACE_GRAY8;
-	} else if (uppValue == L"YUV420P" || uppValue == L"420P" || uppValue == L"7") {
-        paramValue =  amf::AMF_SURFACE_YUV420P;
-    } else if(uppValue == L"U8V8" || uppValue == L"8") {
-        paramValue =  amf::AMF_SURFACE_U8V8;
-    } else if(uppValue == L"YUY2" || uppValue == L"9") {
-        paramValue =  amf::AMF_SURFACE_YUY2;
-    } else if(uppValue == L"P010" || uppValue == L"10") {
-        paramValue =  amf::AMF_SURFACE_P010;
-    } else if (uppValue == L"P012" || uppValue == L"12") {
-        paramValue = amf::AMF_SURFACE_P012;
-    }else if (uppValue == L"P016" || uppValue == L"16") {
-        paramValue = amf::AMF_SURFACE_P016;
-    } else if(uppValue == L"RGBAF16" || uppValue == L"RGBA_F16" || uppValue == L"11") {
-        paramValue =  amf::AMF_SURFACE_RGBA_F16;
-    } else if(uppValue == L"UYVY" || uppValue == L"12") {
-        paramValue =  amf::AMF_SURFACE_UYVY;
-    }else if (uppValue == L"R10G10B10A2" || uppValue == L"13") {
         paramValue = amf::AMF_SURFACE_R10G10B10A2;
-    } else {
-        
-            LOG_ERROR(L"AMF_SURFACE_FORMAT hasn't \"" << value << L"\" value.");
+    }
+    else if (uppValue == L"Y210" || uppValue == L"14")
+    {
+        paramValue = amf::AMF_SURFACE_Y210;
+    }
+    else if (uppValue == L"AYUV" || uppValue == L"15")
+    {
+        paramValue = amf::AMF_SURFACE_AYUV;
+    }
+    else if (uppValue == L"Y410" || uppValue == L"16")
+    {
+        paramValue = amf::AMF_SURFACE_Y410;
+    }
+    else if (uppValue == L"Y416" || uppValue == L"17")
+    {
+        paramValue = amf::AMF_SURFACE_Y416;
+    }
+    else if (uppValue == L"GRAY32" || uppValue == L"18")
+    {
+        paramValue = amf::AMF_SURFACE_GRAY32;
+    }
+    else if (uppValue == L"P012" || uppValue == L"19")
+    {
+        paramValue = amf::AMF_SURFACE_P012;
+    }
+    else if (uppValue == L"P016" || uppValue == L"20")
+    {
+        paramValue = amf::AMF_SURFACE_P016;
+    }
+    else if (uppValue == L"Y216" || uppValue == L"21")
+    {
+        paramValue = amf::AMF_SURFACE_Y216;
+    }
+    else {
+        LOG_ERROR(L"AMF_SURFACE_FORMAT hasn't \"" << value << L"\" value.");
         return AMF_INVALID_ARG;
     }
     valueOut = amf_int64(paramValue);
@@ -341,6 +405,12 @@ AMF_RESULT ParamConverterColorProfile(const std::wstring& value, amf::AMFVariant
         paramValue =  AMF_VIDEO_CONVERTER_COLOR_PROFILE_2020;
     } else if(uppValue == L"JPEG" || uppValue == L"3") {
         paramValue =  AMF_VIDEO_CONVERTER_COLOR_PROFILE_JPEG;
+    } else if(uppValue == L"FULL_601" || uppValue == L"3") {
+        paramValue =  AMF_VIDEO_CONVERTER_COLOR_PROFILE_FULL_601;
+    } else if (uppValue == L"FULL_709" || uppValue == L"7") {
+        paramValue = AMF_VIDEO_CONVERTER_COLOR_PROFILE_FULL_709;
+    } else if (uppValue == L"FULL_2020" || uppValue == L"8") {
+        paramValue = AMF_VIDEO_CONVERTER_COLOR_PROFILE_FULL_2020;
     } else {
         LOG_ERROR(L"AMF_VIDEO_CONVERTER_COLOR_PROFILE_ENUM hasn't \"" << value << L"\" value.");
         return AMF_INVALID_ARG;
@@ -381,7 +451,7 @@ AMF_RESULT ParamConverterTransferCharacteristic(const std::wstring& value, amf::
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_IEC61966_2_1;
 	} else if (uppValue == L"BT2020_10" || uppValue == L"14") {
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_BT2020_10;
-	} else if (uppValue == L"BT2020_10" || uppValue == L"15") {
+	} else if (uppValue == L"BT2020_12" || uppValue == L"15") {
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_BT2020_12;
 	} else if (uppValue == L"SMPTE2084" || uppValue == L"16") {
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_SMPTE2084;
@@ -469,6 +539,30 @@ AMF_RESULT ParamConverterColorRange(const std::wstring& value, amf::AMFVariant&
 	return AMF_OK;
 }
 
+AMF_RESULT ParamConverterToneMapping(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_VIDEO_CONVERTER_TONEMAPPING_ENUM paramValue = AMF_VIDEO_CONVERTER_TONEMAPPING_AMD;
+    std::wstring uppValue = toUpper(value);
+    if (uppValue == L"COPY" || uppValue == L"0") {
+        paramValue = AMF_VIDEO_CONVERTER_TONEMAPPING_COPY;
+    } else if (uppValue == L"AMD" || uppValue == L"1") {
+        paramValue = AMF_VIDEO_CONVERTER_TONEMAPPING_AMD;
+    } else if (uppValue == L"LINEAR" || uppValue == L"2") {
+        paramValue = AMF_VIDEO_CONVERTER_TONEMAPPING_LINEAR;
+    } else if (uppValue == L"GAMMA" || uppValue == L"3") {
+        paramValue = AMF_VIDEO_CONVERTER_TONEMAPPING_GAMMA;
+    } else if (uppValue == L"REINHARD" || uppValue == L"4") {
+        paramValue = AMF_VIDEO_CONVERTER_TONEMAPPING_REINHARD;
+    } else if (uppValue == L"2390" || uppValue == L"5") {
+        paramValue = AMF_VIDEO_CONVERTER_TONEMAPPING_2390;
+    } else {
+        LOG_ERROR(L"AMF_VIDEO_CONVERTER_TONEMAPPING_ENUM hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
+
 AMF_RESULT ParamConverterBoolean(const std::wstring& value, amf::AMFVariant& valueOut)
 {
     bool paramValue = true; // if parameter is present default is true
@@ -501,9 +595,9 @@ AMF_RESULT ParamConverterRate(const std::wstring& value, amf::AMFVariant& valueO
     amf::AMFVariant valueIn(value.c_str());
 
     AMFVariantChangeType(&valueOut, &valueIn, amf::AMF_VARIANT_RATE);
-    if (valueIn.rateValue.den == 0)
+    if (valueOut.rateValue.den == 0)
     {
-        valueIn.rateValue.den = 1;
+        valueOut.rateValue.den = 1;
     }
 
     return AMF_OK;
@@ -568,7 +662,7 @@ AMF_RESULT ParamConverterCodec(const std::wstring& value, amf::AMFVariant& value
     else if (uppValue == L"AVC" || uppValue == L"H264" || uppValue == L"H.264")
     {
         paramValue = AMFVideoEncoderVCE_AVC;
-    } 
+    }
     else if(uppValue == L"HEVC" || uppValue == L"H265" || uppValue == L"H.265")
     {
         paramValue = AMFVideoEncoder_HEVC;
@@ -577,7 +671,7 @@ AMF_RESULT ParamConverterCodec(const std::wstring& value, amf::AMFVariant& value
     {
         paramValue = AMFVideoEncoder_AV1;
     }
-    else 
+    else
     {
         LOG_ERROR(L"Invalid codec name \"" << value << L"\" value.");
         return AMF_INVALID_ARG;
@@ -592,12 +686,12 @@ const wchar_t *StreamCodecIDtoDecoderID(AMF_STREAM_CODEC_ID_ENUM eCodec)
     {
     case AMF_STREAM_CODEC_ID_UNKNOWN: return L"";
     case AMF_STREAM_CODEC_ID_MPEG2: return AMFVideoDecoderUVD_MPEG2;
-    case AMF_STREAM_CODEC_ID_MPEG4: return AMFVideoDecoderUVD_MPEG4;      
-    case AMF_STREAM_CODEC_ID_WMV3: return AMFVideoDecoderUVD_WMV3;       
-    case AMF_STREAM_CODEC_ID_VC1: return AMFVideoDecoderUVD_VC1;        
+    case AMF_STREAM_CODEC_ID_MPEG4: return AMFVideoDecoderUVD_MPEG4;
+    case AMF_STREAM_CODEC_ID_WMV3: return AMFVideoDecoderUVD_WMV3;
+    case AMF_STREAM_CODEC_ID_VC1: return AMFVideoDecoderUVD_VC1;
     case AMF_STREAM_CODEC_ID_H264_AVC: return AMFVideoDecoderUVD_H264_AVC;
-    case AMF_STREAM_CODEC_ID_H264_MVC: return AMFVideoDecoderUVD_H264_MVC;  
-    case AMF_STREAM_CODEC_ID_H264_SVC: return AMFVideoDecoderUVD_H264_SVC;  
+    case AMF_STREAM_CODEC_ID_H264_MVC: return AMFVideoDecoderUVD_H264_MVC;
+    case AMF_STREAM_CODEC_ID_H264_SVC: return AMFVideoDecoderUVD_H264_SVC;
     case AMF_STREAM_CODEC_ID_MJPEG: return AMFVideoDecoderUVD_MJPEG;
     case AMF_STREAM_CODEC_ID_H265_HEVC: return AMFVideoDecoderHW_H265_HEVC;
     case AMF_STREAM_CODEC_ID_H265_MAIN10: return AMFVideoDecoderHW_H265_MAIN10;
@@ -764,6 +858,108 @@ AMF_RESULT ParamConverterPAQMode(const std::wstring& value, amf::AMFVariant& val
         LOG_ERROR(L"AMF_PA_PAQ_MODE_ENUM hasn't \"" << value << L"\" value.");
         return AMF_INVALID_ARG;
     }
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
+
+AMF_RESULT ParamConverterFRCEngine(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_FRC_ENGINE    paramValue;
+    std::wstring uppValue = toUpper(value);
+    if ((uppValue == L"OFF") || (uppValue == L"0"))
+    {
+        paramValue = FRC_ENGINE_OFF;
+    }
+    else if ((uppValue == L"DX12") || (uppValue == L"1"))
+    {
+        paramValue = FRC_ENGINE_DX12;
+    }
+    else if ((uppValue == L"OPENCL") || (uppValue == L"2"))
+    {
+        paramValue = FRC_ENGINE_OPENCL;
+    }
+    else if ((uppValue == L"DX11") || (uppValue == L"3"))
+    {
+        paramValue = FRC_ENGINE_DX11;
+    }
+    else {
+        LOG_ERROR(L"AMF_FRC_ENGINE hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
+
+AMF_RESULT ParamConverterFRCMode(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_FRC_MODE_TYPE    paramValue;
+    std::wstring uppValue = toUpper(value);
+    if ((uppValue == L"OFF") || (uppValue == L"0"))
+    {
+        paramValue = FRC_OFF;
+    }
+    else if ((uppValue == L"ON") || (uppValue == L"1"))
+    {
+        paramValue = FRC_ON;
+    }
+    else if ((uppValue == L"INTERPOLATED") || (uppValue == L"2"))
+    {
+        paramValue = FRC_ONLY_INTERPOLATED;
+    }
+    else if ((uppValue == L"PRESENT") || (uppValue == L"3"))
+    {
+        paramValue = FRC_x2_PRESENT;
+    }
+    else {
+        LOG_ERROR(L"AMF_FRC_MODE_TYPE hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
+
+AMF_RESULT ParamConverterFRCPerformance(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_FRC_MV_SEARCH_MODE_TYPE    paramValue;
+    std::wstring uppValue = toUpper(value);
+    if ((uppValue == L"NATIVE") || (uppValue == L"0"))
+    {
+        paramValue = FRC_MV_SEARCH_NATIVE;
+    }
+    else if ((uppValue == L"PERFORMANCE") || (uppValue == L"1"))
+    {
+        paramValue = FRC_MV_SEARCH_PERFORMANCE;
+    }
+    else {
+        LOG_ERROR(L"AMF_FRC_MV_SEARCH_MODE_TYPE hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+
+    valueOut = amf_int64(paramValue);
+    return AMF_OK;
+}
+AMF_RESULT ParamConverterFRCProfile(const std::wstring& value, amf::AMFVariant& valueOut)
+{
+    AMF_FRC_PROFILE_TYPE    paramValue;
+    std::wstring uppValue = toUpper(value);
+    if ((uppValue == L"LOW") || (uppValue == L"0"))
+    {
+        paramValue = FRC_PROFILE_LOW;
+    }
+    else if ((uppValue == L"HIGH") || (uppValue == L"1"))
+    {
+        paramValue = FRC_PROFILE_HIGH;
+    }
+    else if ((uppValue == L"SUPER") || (uppValue == L"2"))
+    {
+        paramValue = FRC_PROFILE_SUPER;
+    }
+    else {
+        LOG_ERROR(L"AMF_FRC_PROFILE_TYPE hasn't \"" << value << L"\" value.");
+        return AMF_INVALID_ARG;
+    }
+
     valueOut = amf_int64(paramValue);
     return AMF_OK;
 }
\ No newline at end of file
