@@ -60,6 +60,15 @@ const wchar_t* PlaybackPipelineBase::PARAM_NAME_HQ_SCALER_RATIO = L"HQSCALERRATI
 const wchar_t* PlaybackPipelineBase::PARAM_NAME_ENABLE_AUDIO = L"ENABLEAUDIO";
 const wchar_t* PlaybackPipelineBase::PARAM_NAME_HQSCALER_SHARPNESS = L"HQSCALERSHARPNESS";
 const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRAME_RATE = L"FRAMERATE";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_ENGINE = L"FRCENGINE";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_INFO = L"FRCINFO";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_MODE = L"FRCMODE";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_ENABLE_FALLBACK = L"FRCENABLEFALLBACK";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_INDICATOR = L"FRCINDICATOR";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_EXCLUSIVE_FULLSCREEN = L"EXCLUSIVE_FULLSCREEN";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_PROFILE = L"FRCPROFILE";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_MV_SEARCH_MODE = L"FRCPERFORMANCE";
+const wchar_t* PlaybackPipelineBase::PARAM_NAME_FRC_RGB = L"FRCRGB";
 
 PlaybackPipelineBase::PlaybackPipelineBase() :
     m_iVideoWidth(0),
@@ -91,12 +100,21 @@ PlaybackPipelineBase::PlaybackPipelineBase() :
     SetParamDescription(PARAM_NAME_SIDE_BY_SIDE, ParamCommon, L"Specifies Side-by-Side mode, true, false, default false", ParamConverterBoolean);
     SetParamDescription(PARAM_NAME_HQ_SCALER_RGB, ParamCommon, L"Force RGB scaling, true, false, default false", ParamConverterBoolean);
     SetParamDescription(PARAM_NAME_HQ_SCALER_RATIO, ParamCommon, L"Scaling Ratio (1.3x, 1.5x, 1.7x, 2.0x), default 2.0x", ParamConverterRatio);
-    SetParamDescription(PARAM_NAME_ENABLE_AUDIO, ParamCommon, L"Enables audio playback, boolean, default = true", ParamConverterDouble);
-    SetParamDescription(PARAM_NAME_HQSCALER_SHARPNESS, ParamCommon, L"Specifies FSR RCAS attenuation, double, default 0.75", ParamConverterDouble);
-    SetParamDescription(PARAM_NAME_FRAME_RATE, ParamCommon, L"Frame Rate (off, 15fps, 30fps, 60fps), default off", ParamConverterInt64);
+    SetParamDescription(PARAM_NAME_ENABLE_AUDIO, ParamCommon, L"Enables audio playback, boolean, default = true", ParamConverterBoolean);
+    SetParamDescription(PARAM_NAME_HQSCALER_SHARPNESS, ParamCommon, L"Specifies FSR RCAS attenuation, double, default = 0.75", ParamConverterDouble);
+    SetParamDescription(PARAM_NAME_FRAME_RATE, ParamCommon, L"Frame Rate (off, 15fps, 30fps, 60fps), default = off", ParamConverterInt64);
+    SetParamDescription(PARAM_NAME_FRC_ENGINE, ParamCommon, L"Frame Rate Conversion, (DX11, DX12) default = from '-presenter'", ParamConverterFRCEngine);
+    SetParamDescription(PARAM_NAME_FRC_MODE, ParamCommon, L"FRC Mode, (OFF, ON, INTERPOLATED, x2_PRESENT) default = OFF ", ParamConverterFRCMode);
+    SetParamDescription(PARAM_NAME_FRC_ENABLE_FALLBACK, ParamCommon, L"FRC enable fallback, (true, false), default = false", ParamConverterBoolean);
+    SetParamDescription(PARAM_NAME_FRC_INDICATOR, ParamCommon, L"FRC Indicator, (true, false),  default = false", ParamConverterBoolean);
+    SetParamDescription(PARAM_NAME_EXCLUSIVE_FULLSCREEN, ParamCommon, L"Specifies exclusive fullscreen mode, (true, false), default true", ParamConverterBoolean);
+    SetParamDescription(PARAM_NAME_FRC_PROFILE, ParamCommon, L"FRC Profile,  default High", ParamConverterFRCProfile);
+    SetParamDescription(PARAM_NAME_FRC_MV_SEARCH_MODE, ParamCommon, L"FRC Performance,  (NATIVE, PERFORMANCE) default = NATIVE", ParamConverterFRCPerformance);
+    SetParamDescription(PARAM_NAME_FRC_RGB, ParamCommon, L"Run FRC in RGB or YUV node, true, false, default = false", ParamConverterBoolean);
 
     SetParam(PARAM_NAME_LISTEN_FOR_CONNECTION, false);
     SetParam(PARAM_NAME_FULLSCREEN, false);
+    SetParam(PARAM_NAME_EXCLUSIVE_FULLSCREEN, true);
     SetParam(PARAM_NAME_PIP, false);
     SetParam(PARAM_NAME_PIP_ZOOM_FACTOR, 0.4f);
     SetParam(PARAM_NAME_PIP_FOCUS_X, 0);
@@ -107,7 +125,6 @@ PlaybackPipelineBase::PlaybackPipelineBase() :
     SetParam(PARAM_NAME_ENABLE_AUDIO, true);
     SetParam(PARAM_NAME_HQSCALER_SHARPNESS, 0.75);
     SetParam(PARAM_NAME_FRAME_RATE, 0);
-
 #if defined(_WIN32)
     SetParam(PlaybackPipelineBase::PARAM_NAME_PRESENTER, amf::AMF_MEMORY_DX11);
 #elif defined(__linux)
@@ -129,11 +146,6 @@ PlaybackPipelineBase::~PlaybackPipelineBase()
 void PlaybackPipelineBase::Terminate()
 {
     Stop();
-    if (m_pContext != nullptr)
-    {
-        m_pContext->Terminate();
-        m_pContext = NULL;
-    }
 }
 
 AMF_RESULT PlaybackPipelineBase::GetDuration(amf_pts& duration) const
@@ -279,12 +291,11 @@ AMF_RESULT PlaybackPipelineBase::Init()
 	amf::AMF_MEMORY_TYPE presenterEngine = amf::AMF_MEMORY_UNKNOWN;
     {
         amf_int64 engineInt = amf::AMF_MEMORY_UNKNOWN;
-        if(GetParam(PARAM_NAME_PRESENTER, engineInt) == AMF_OK)
+        GetParam(PARAM_NAME_PRESENTER, engineInt);
+
+        if(amf::AMF_MEMORY_UNKNOWN != engineInt)
         {
-            if(amf::AMF_MEMORY_UNKNOWN != engineInt)
-            {
-                presenterEngine = (amf::AMF_MEMORY_TYPE)engineInt;
-            }
+            presenterEngine = (amf::AMF_MEMORY_TYPE)engineInt;
         }
     }
 
@@ -309,7 +320,7 @@ AMF_RESULT PlaybackPipelineBase::Init()
 
     amf::AMFOutputPtr       pAudioOutput;
     amf::AMFOutputPtr       pVideoOutput;
-    if( streamType != BitStreamUnknown)
+    if( streamType != BitStreamUnknown && streamType != BitStreamIVF)
     {
 #if !defined(METRO_APP)
         amf::AMFDataStream::OpenDataStream(inputPath.c_str(), amf::AMFSO_READ, amf::AMFFS_SHARE_READ, &m_pVideoStream);
@@ -429,6 +440,23 @@ AMF_RESULT PlaybackPipelineBase::Init()
         {
             LOG_AMF_ERROR(res, L"InitAudio() failed");
             iAudioStreamIndex = -1;
+            if (m_pAudioDecoder != NULL)
+            {
+                m_pAudioDecoder->Terminate();
+                m_pAudioDecoder = NULL;
+            }
+
+            if (m_pAudioConverter != NULL)
+            {
+                m_pAudioConverter->Terminate();
+                m_pAudioConverter = NULL;
+            }
+
+            if (m_pAudioPresenter != NULL)
+            {
+                m_pAudioPresenter = NULL;
+            }
+
         }
 //        CHECK_AMF_ERROR_RETURN(res, L"InitAudio() failed");
     }
@@ -490,7 +518,7 @@ AMF_RESULT PlaybackPipelineBase::Init()
 
 AMF_RESULT PlaybackPipelineBase::InitAudioPipeline(amf_uint32 iAudioStreamIndex, PipelineElementPtr pAudioSourceStream)
 {
-	if (iAudioStreamIndex >= 0 && m_pAudioPresenter != NULL && pAudioSourceStream != NULL)
+	if (m_pAudioPresenter != NULL && pAudioSourceStream != NULL)
 	{
 		Connect(PipelineElementPtr(new AMFComponentElement(m_pAudioDecoder)), 0, pAudioSourceStream, iAudioStreamIndex, m_bURL ? 1000 : 100, CT_ThreadQueue);
 		Connect(PipelineElementPtr(new AMFComponentElement(m_pAudioConverter)), 10);
@@ -502,34 +530,15 @@ AMF_RESULT PlaybackPipelineBase::InitAudioPipeline(amf_uint32 iAudioStreamIndex,
 	return AMF_OK;
 }
 
-AMF_RESULT PlaybackPipelineBase::InitVideoPipeline(amf_uint32 /* iVideoStreamIndex */, PipelineElementPtr pVideoSourceStream)
+AMF_RESULT  PlaybackPipelineBase::ConnectScaler()
 {
-    bool bLowlatency = false;
-    GetParam(PARAM_NAME_LOWLATENCY, bLowlatency);
-
-    bool bForceScalingRGB = false;
-    GetParam(PARAM_NAME_HQ_SCALER_RGB, bForceScalingRGB);
-    bool bYUVScaling = (((m_eDecoderFormat == amf::AMF_SURFACE_NV12) || (m_eDecoderFormat == amf::AMF_SURFACE_P010)) &&
-                        (m_pScaler != nullptr) &&
-                        !bForceScalingRGB);
-
-    if (!bYUVScaling && (m_pVideoProcessor != NULL))
-	{
-        if (bLowlatency)
-        {
-            Connect(PipelineElementPtr(new AMFComponentElement(m_pVideoProcessor)), 1, CT_Direct);
-        }
-        else
-        {
-            Connect(PipelineElementPtr(new AMFComponentElement(m_pVideoProcessor)), m_bCPUDecoder ? 4 : 1, CT_ThreadQueue);
-        }
-    }
-
+    AMF_RESULT res = AMF_FAIL;
     if (m_pScaler != NULL)
     {
         if (m_pHQScaler2 == NULL)
         {
-            Connect(PipelineElementPtr(new AMFComponentElement(m_pScaler)), 1, CT_Direct);
+            Connect(PipelineElementPtr(new AMFComponentElement(m_pScaler)), 1, CT_ThreadQueue);
+            res = AMF_OK;
         }
         else
         {
@@ -556,21 +565,79 @@ AMF_RESULT PlaybackPipelineBase::InitVideoPipeline(amf_uint32 /* iVideoStreamInd
 
                 Connect(m_pCombiner, 0, hqScalerElement1, 0, 4);
                 Connect(m_pCombiner, 1, hqScalerElement2, 0, 4);
+                res = AMF_OK;
             }
         }
     }
+    return res;
+}
+AMF_RESULT PlaybackPipelineBase::InitVideoPipeline(amf_uint32 /* iVideoStreamIndex */, PipelineElementPtr /*pVideoSourceStream*/)
+{
+    bool bAllocator = true;
 
-    if (bYUVScaling && (m_pVideoProcessor != NULL))
+    bool bLowlatency = false;
+    GetParam(PARAM_NAME_LOWLATENCY, bLowlatency);
+
+    bool bForceScalingRGB = false;
+    GetParam(PARAM_NAME_HQ_SCALER_RGB, bForceScalingRGB);
+    bool bForceFRCRGB = false;
+    GetParam(PARAM_NAME_FRC_RGB, bForceFRCRGB);
+
+    bool bYUVInput = (m_eDecoderFormat == amf::AMF_SURFACE_YUV420P) || (m_eDecoderFormat == amf::AMF_SURFACE_NV12) || (m_eDecoderFormat == amf::AMF_SURFACE_P010) || (m_eDecoderFormat == amf::AMF_SURFACE_P012) || (m_eDecoderFormat == amf::AMF_SURFACE_P016);
+
+    if (bYUVInput == false)
     {
+        bForceScalingRGB = true;
+        bForceFRCRGB = true;
+    }
+
+//    bool bYUVScaling = (((m_eDecoderFormat == amf::AMF_SURFACE_NV12) || (m_eDecoderFormat == amf::AMF_SURFACE_P010)) &&
+//                        ((m_pFRC != nullptr) ||
+//                         (m_pScaler != nullptr)) &&
+//                        !bForceScalingRGB);
+
+    // connect YUV
+    if (m_pFRC != NULL && bForceFRCRGB == false) //FRC
+    {
+        Connect(PipelineElementPtr(new AMFComponentElement(m_pFRC)), 1, CT_ThreadQueue);
+    }
+    // connect scaler
+    if (bForceScalingRGB == false && m_pScaler != nullptr)
+    {
+        ConnectScaler();
+        bAllocator = false;
+    }
+    // connect color converter
+    if (m_pVideoConverter != NULL)
+	{
         if (bLowlatency)
         {
-            Connect(PipelineElementPtr(new AMFComponentElement(m_pVideoProcessor)), 1, CT_Direct);
+            Connect(PipelineElementPtr(new AMFComponentElement(m_pVideoConverter)), 1, CT_Direct);
         }
         else
         {
-            Connect(PipelineElementPtr(new AMFComponentElement(m_pVideoProcessor)), m_bCPUDecoder ? 4 : 1, CT_ThreadQueue);
+            Connect(PipelineElementPtr(new AMFComponentElement(m_pVideoConverter)), m_bCPUDecoder ? 4 : 1, CT_ThreadQueue);
         }
     }
+    // connect RGB
+    if (m_pFRC != NULL && bForceFRCRGB == true) //FRC
+    {
+        Connect(PipelineElementPtr(new AMFComponentElement(m_pFRC)), 1, CT_ThreadQueue);
+        bAllocator = false;
+    }
+    // connect scaler
+    if (bForceScalingRGB == true && m_pScaler != nullptr)
+    {
+        ConnectScaler();
+        amf_int64  hqScalerMode = -1;
+        GetParam(PARAM_NAME_HQ_SCALER, hqScalerMode);
+        bAllocator = (hqScalerMode != -1); // no allocator non HQ scaler is setup by calling app
+
+    }
+    // connect presenter
+
+    m_pVideoPresenter->SetProcessor(m_pVideoConverter, m_pScaler, bAllocator);
+
     if(m_bVideoPresenterDirectConnect)
     {
         Connect(m_pVideoPresenter, 4, CT_ThreadPoll); // if back buffers are used in multithreading the video memory changes every Present() call so pointer returned by GetBackBuffer() points to wrong memory
@@ -582,14 +649,140 @@ AMF_RESULT PlaybackPipelineBase::InitVideoPipeline(amf_uint32 /* iVideoStreamInd
 	return AMF_OK;
 }
 
+AMF_RESULT  PlaybackPipelineBase::InitFRC(amf::AMF_MEMORY_TYPE type)
+{
+    AMF_RESULT res = AMF_OK;
+    amf_int64 frcMode = FRC_OFF;
+    GetParam(PARAM_NAME_FRC_MODE, frcMode);
+
+    if (frcMode != FRC_OFF)
+    {
+        amf_int64 frcEngineType = FRC_ENGINE_OFF;
+        GetParam(PlaybackPipelineBase::PARAM_NAME_FRC_ENGINE, frcEngineType);
+        if (frcEngineType == FRC_ENGINE_OFF)
+        {
+            switch (type)
+            {
+            case amf::AMF_MEMORY_OPENCL: frcEngineType = FRC_ENGINE_OPENCL; break;
+            case amf::AMF_MEMORY_DX11: frcEngineType = FRC_ENGINE_DX11; break;
+            case amf::AMF_MEMORY_DX12: frcEngineType = FRC_ENGINE_DX12; break;
+            default:
+                frcEngineType = FRC_ENGINE_DX11; break;
+            }
+        }
+        bool bForceFRCRGB = false;
+        GetParam(PARAM_NAME_FRC_RGB, bForceFRCRGB);
+        bool bYUVInput = (m_eDecoderFormat == amf::AMF_SURFACE_NV12) || (m_eDecoderFormat == amf::AMF_SURFACE_P010) || (m_eDecoderFormat == amf::AMF_SURFACE_P012) || (m_eDecoderFormat == amf::AMF_SURFACE_P016);
+        if (bYUVInput == false)
+        {
+            if (frcEngineType == FRC_ENGINE_OPENCL)
+            {
+                AMFTraceWarning(AMF_FACILITY, L"FRC not supported for OpenCL + RGB input, skipping FRC");
+                return AMF_OK;
+            }
+            bForceFRCRGB = true;
+        }
+        if (frcEngineType == FRC_ENGINE_OPENCL && bForceFRCRGB == true)
+        {
+            AMFTraceInfo(AMF_FACILITY, L"FRC not supported for OpenCL + RGB, switching to YUV");
+            SetParam(PARAM_NAME_FRC_RGB, false);
+            bForceFRCRGB = false;
+        }
+
+        res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFFRC, &m_pFRC);
+        CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFFRC << L") failed");
+        //m_pFRC->SetProperty(AMF_HQ_SCALER_ENGINE_TYPE, type);
+
+        // initialize
+        if (m_pFRC != nullptr)
+        {
+            SetParam(PARAM_NAME_HQ_SCALER_RATIO, AMFRatio({ 10, 10 }));
+
+            m_pFRC->SetProperty(AMF_FRC_ENGINE_TYPE, frcEngineType);
+
+            m_pFRC->SetProperty(AMF_FRC_MODE, frcMode);
+
+            amf_int64 frcFlag = 0;
+
+            GetParam(PlaybackPipelineBase::PARAM_NAME_FRC_ENABLE_FALLBACK, frcFlag);
+            m_pFRC->SetProperty(AMF_FRC_ENABLE_FALLBACK, frcFlag);
+
+            bool indicator = false;
+            GetParam(PlaybackPipelineBase::PARAM_NAME_FRC_INDICATOR, indicator);
+            m_pFRC->SetProperty(AMF_FRC_INDICATOR, indicator);
+
+            amf_int64 frcProfile = FRC_PROFILE_HIGH;
+            GetParam(PlaybackPipelineBase::PARAM_NAME_FRC_PROFILE, frcProfile);
+            m_pFRC->SetProperty(AMF_FRC_PROFILE, frcProfile);
+
+            amf_int64 frcMVSearchMode = FRC_MV_SEARCH_NATIVE;
+            GetParam(PlaybackPipelineBase::PARAM_NAME_FRC_MV_SEARCH_MODE, frcMVSearchMode);
+            m_pFRC->SetProperty(AMF_FRC_MV_SEARCH_MODE, frcMVSearchMode);
+
+            res = m_pFRC->Init(bForceFRCRGB == true ? m_pVideoPresenter->GetInputFormat() : m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
+            CHECK_AMF_ERROR_RETURN(res, L"m_pFRC->Init() failed with err=%d");
+
+            if (bForceFRCRGB == true) //FRC is last filter, but doesn't support scaling
+            {
+                m_pVideoPresenter->SetRenderToBackBuffer(false);
+            }
+        }
+    }
+    return res;
+}
+
+AMF_RESULT  PlaybackPipelineBase::InitColorConverter()
+{
+    AMF_RESULT res = AMF_OK;
+    // create CSC - this one is needed all the time,
+    if (m_pVideoConverter == nullptr)
+    {
+        res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFVideoConverter, &m_pVideoConverter);
+        CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFVideoConverter << L") failed");
+
+        if (m_eDecoderFormat == amf::AMF_SURFACE_P010) //HDR support
+        {
+            //        m_pVideoConverter->SetProperty(AMF_VIDEO_CONVERTER_COLOR_PROFILE, AMF_VIDEO_CONVERTER_COLOR_PROFILE_2020);
+            //        m_pVideoConverter->SetProperty(AMF_VIDEO_CONVERTER_LINEAR_RGB, true);
+        }
+        else
+        {
+            //        m_pVideoConverter->SetProperty(AMF_VIDEO_CONVERTER_COLOR_PROFILE, AMF_VIDEO_CONVERTER_COLOR_PROFILE_709);
+        }
+        m_pVideoConverter->SetProperty(AMF_VIDEO_CONVERTER_MEMORY_TYPE, m_pVideoPresenter->GetMemoryType());
+        m_pVideoConverter->SetProperty(AMF_VIDEO_CONVERTER_OUTPUT_FORMAT, m_pVideoPresenter->GetInputFormat());
+
+        res = m_pVideoConverter->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
+        CHECK_AMF_ERROR_RETURN(res, L"m_pVideoConverter->Init() failed");
+
+
+        // set-up the processor for presenter - if we use HQ scaler,
+        // we need to set it, otherwise it will be CSC
+
+        if (m_pContext->GetOpenCLContext() == NULL)
+        {
+            m_pVideoConverter->SetProperty(AMF_VIDEO_CONVERTER_KEEP_ASPECT_RATIO, true);
+            m_pVideoConverter->SetProperty(AMF_VIDEO_CONVERTER_FILL, true);
+
+            if (m_pScaler != nullptr)
+            {
+                m_pScaler->SetProperty(AMF_HQ_SCALER_KEEP_ASPECT_RATIO, true);
+                m_pScaler->SetProperty(AMF_HQ_SCALER_FILL, true);
+            }
+        }
+    }
+    return res;
+
+}
 AMF_RESULT  PlaybackPipelineBase::InitVideoProcessor()
 {
     // check if we need to create the HQ scaler
+    AMF_RESULT res = AMF_OK;
     amf_int64  hqScalerMode = -1;
 
-    AMF_RESULT res = GetParam(PARAM_NAME_HQ_SCALER, hqScalerMode);
+    GetParam(PARAM_NAME_HQ_SCALER, hqScalerMode);
 
-    if ((res == AMF_OK) && (hqScalerMode != -1))
+    if (hqScalerMode != -1)
     {
         res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFHQScaler, &m_pScaler);
         CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFHQScaler << L") failed");
@@ -628,53 +821,18 @@ AMF_RESULT  PlaybackPipelineBase::InitVideoProcessor()
         }
         */
 
-    }
-
-    // initialize scaler
-    if (m_pScaler != nullptr)
-    {
+        // initialize scaler
         bool bForceScalingRGB = false;
         GetParam(PARAM_NAME_HQ_SCALER_RGB, bForceScalingRGB);
-        res = m_pScaler->Init(bForceScalingRGB ? m_pVideoPresenter->GetInputFormat() : m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
-        CHECK_AMF_ERROR_RETURN(res, L"m_pScaler->Init() failed with err=%d");
-    }
 
-    // create CSC - this one is needed all the time,
-    // even if we use HQ scaler
-    res = g_AMFFactory.GetFactory()->CreateComponent(m_pContext, AMFVideoConverter, &m_pVideoProcessor);
-    CHECK_AMF_ERROR_RETURN(res, L"g_AMFFactory.GetFactory()->CreateComponent(" << AMFVideoConverter << L") failed");
-
-    if (m_eDecoderFormat == amf::AMF_SURFACE_P010) //HDR support
-    {
-//        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_COLOR_PROFILE, AMF_VIDEO_CONVERTER_COLOR_PROFILE_2020);
-//        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_LINEAR_RGB, true);
-    }
-    else
-    {
-//        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_COLOR_PROFILE, AMF_VIDEO_CONVERTER_COLOR_PROFILE_709);
-    }
-    m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_MEMORY_TYPE, m_pVideoPresenter->GetMemoryType());
-    m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_OUTPUT_FORMAT, m_pVideoPresenter->GetInputFormat());
-
-    res = m_pVideoProcessor->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
-    CHECK_AMF_ERROR_RETURN(res, L"m_pVideoProcessor->Init() failed");
-
-
-    // set-up the processor for presenter - if we use HQ scaler,
-    // we need to set it, otherwise it will be CSC
-
-    if (m_pContext->GetOpenCLContext() == NULL)
-    {
-        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_KEEP_ASPECT_RATIO, true);
-        m_pVideoProcessor->SetProperty(AMF_VIDEO_CONVERTER_FILL, true);
-
-        if (m_pScaler != nullptr)
+        bool bYUVInput = (m_eDecoderFormat == amf::AMF_SURFACE_NV12) || (m_eDecoderFormat == amf::AMF_SURFACE_P010) || (m_eDecoderFormat == amf::AMF_SURFACE_P012) || (m_eDecoderFormat == amf::AMF_SURFACE_P016);
+        if (bYUVInput == false)
         {
-            m_pScaler->SetProperty(AMF_HQ_SCALER_KEEP_ASPECT_RATIO, true);
-            m_pScaler->SetProperty(AMF_HQ_SCALER_FILL, true);
+            bForceScalingRGB = true;
         }
 
-        m_pVideoPresenter->SetProcessor(m_pVideoProcessor, m_pScaler);
+        res = m_pScaler->Init(bForceScalingRGB ? m_pVideoPresenter->GetInputFormat() : m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
+        CHECK_AMF_ERROR_RETURN(res, L"m_pScaler->Init() failed with err=%d");
     }
 
     return res;
@@ -715,7 +873,7 @@ AMF_RESULT  PlaybackPipelineBase::InitVideoDecoder(const wchar_t *pDecoderID, am
 
     // switch to SW Decoder when the HW Decoder does not support this config
     amf::AMFCapsPtr pCaps;
-    if (m_pVideoDecoder != nullptr && 
+    if (m_pVideoDecoder != nullptr &&
         m_pVideoDecoder->GetCaps(&pCaps) != AMF_OK)
     {
         m_pVideoDecoder = NULL;
@@ -745,11 +903,18 @@ AMF_RESULT  PlaybackPipelineBase::InitVideoDecoder(const wchar_t *pDecoderID, am
             {
                 m_eDecoderFormat = amf::AMF_SURFACE_P010;
             }
-            if (std::wstring(pDecoderID) == AMFVideoDecoderHW_AV1)
+            if (std::wstring(pDecoderID) == AMFVideoDecoderHW_AV1 || std::wstring(pDecoderID) == AMFVideoDecoderHW_AV1_12BIT)
             {
                 m_eDecoderFormat = surfaceFormat;
             }
-            res = m_pVideoDecoder->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
+            if (pExtraData != nullptr || std::wstring(pDecoderID) == AMFVideoDecoderUVD_MJPEG)
+            {
+                res = m_pVideoDecoder->Init(m_eDecoderFormat, m_iVideoWidth, m_iVideoHeight);
+            }
+            else
+            {
+                res = m_pVideoDecoder->Init(m_eDecoderFormat, 0, 0);
+            }
             CHECK_AMF_ERROR_RETURN(res, L"m_pVideoDecoder->Init(" << m_iVideoWidth << L", " << m_iVideoHeight << L") failed " << pDecoderID);
 		    if (std::wstring(pDecoderID) == AMFVideoDecoderHW_AV1)
 		    {
@@ -961,10 +1126,10 @@ AMF_RESULT PlaybackPipelineBase::Stop()
         m_pVideoDecoder = NULL;
     }
 
-    if(m_pVideoProcessor != NULL)
+    if(m_pVideoConverter != NULL)
     {
-        m_pVideoProcessor->Terminate();
-        m_pVideoProcessor = NULL;
+        m_pVideoConverter->Terminate();
+        m_pVideoConverter = NULL;
     }
 
     if (m_pScaler != NULL)
@@ -979,7 +1144,13 @@ AMF_RESULT PlaybackPipelineBase::Stop()
         m_pHQScaler2 = NULL;
     }
 
-    if(m_pVideoPresenter != NULL)
+   if (m_pFRC != NULL)
+    {
+       m_pFRC->Terminate();
+       m_pFRC = NULL;
+    }
+
+   if(m_pVideoPresenter != NULL)
     {
         m_pVideoPresenter->Terminate();
         m_pVideoPresenter = NULL;
@@ -997,14 +1168,11 @@ AMF_RESULT PlaybackPipelineBase::Stop()
         m_pDemuxerAudio->Terminate();
         m_pDemuxerAudio = NULL;
     }
-
-
     if(m_pContext != NULL)
     {
         m_pContext->Terminate();
         m_pContext = NULL;
     }
-
     return AMF_OK;
 }
 
@@ -1040,6 +1208,50 @@ void PlaybackPipelineBase::OnParamChanged(const wchar_t* name)
         }
     }
 
+    //check if FRC will need to be re-created in the pipeline
+    if (std::wstring(name) == std::wstring(PARAM_NAME_FRC_ENGINE))
+    {
+        amf_int64  frcEngineType = FRC_ENGINE_OFF;
+        if (m_pFRC != nullptr)
+        {
+            // Running FRC component engine type.
+            m_pFRC->GetProperty(AMF_FRC_ENGINE_TYPE, &frcEngineType);
+        }
+
+        // Pipeline selected FRC engine type.
+        amf_int64  frcEngineTypePipeline = FRC_ENGINE_OFF;
+        GetParam(PARAM_NAME_FRC_ENGINE, frcEngineTypePipeline);
+
+        if (frcEngineTypePipeline != frcEngineType) //need to re-build the pipeline
+        {
+            ReInit();
+        }
+    }
+
+    //update FRC fallback mode
+    if (std::wstring(name) == std::wstring(PARAM_NAME_FRC_ENABLE_FALLBACK))
+    {
+        if (m_pFRC != nullptr)
+        {
+            // Pipeline selected FRC engine type.
+            amf_bool bEnableFallback = false;
+            GetParam(PARAM_NAME_FRC_ENABLE_FALLBACK, bEnableFallback);
+            m_pFRC->SetProperty(AMF_FRC_ENABLE_FALLBACK, bEnableFallback);
+        }
+    }
+
+    //update FRC mode
+    if (std::wstring(name) == std::wstring(PARAM_NAME_FRC_MODE))
+    {
+        if (m_pFRC != nullptr)
+        {
+            // Pipeline selected FRC engine type.
+            amf_int64 frcMode = 0;
+            GetParam(PARAM_NAME_FRC_MODE, frcMode);
+            m_pFRC->SetProperty(AMF_FRC_MODE, frcMode);
+        }
+    }
+
     if (m_pVideoPresenter != nullptr)
     {
         if (std::wstring(name) == std::wstring(PARAM_NAME_FULLSCREEN))
@@ -1048,6 +1260,12 @@ void PlaybackPipelineBase::OnParamChanged(const wchar_t* name)
             GetParam(PARAM_NAME_FULLSCREEN, bFullScreen);
             m_pVideoPresenter->SetFullScreen(bFullScreen);
         }
+        else if (std::wstring(name) == std::wstring(PARAM_NAME_EXCLUSIVE_FULLSCREEN))
+        {
+            bool bExclusive = false;
+            GetParam(PARAM_NAME_EXCLUSIVE_FULLSCREEN, bExclusive);
+            m_pVideoPresenter->SetExclusiveFullscreen(bExclusive);
+        }
         else if (std::wstring(name) == std::wstring(PARAM_NAME_DOTIMING))
         {
             bool bDoTiming = true;
@@ -1125,7 +1343,7 @@ void PlaybackPipelineBase::OnParamChanged(const wchar_t* name)
             }
         }
     }
-    if(m_pVideoProcessor == NULL)
+    if(m_pVideoConverter == NULL)
     {
         return;
     }
@@ -1134,7 +1352,7 @@ void PlaybackPipelineBase::OnParamChanged(const wchar_t* name)
 
 void PlaybackPipelineBase::UpdateVideoProcessorProperties(const wchar_t* name)
 {
-    if (m_pVideoProcessor == NULL)
+    if (m_pVideoConverter == NULL)
     {
         return;
     }
@@ -1147,7 +1365,7 @@ void PlaybackPipelineBase::UpdateVideoProcessorProperties(const wchar_t* name)
         {
             amf::AMFVariant value;
             GetParam(name, &value);
-            m_pVideoProcessor->SetProperty(description.m_Name.c_str(), value);
+            m_pVideoConverter->SetProperty(description.m_Name.c_str(), value);
         }
     }
     else
@@ -1161,7 +1379,7 @@ void PlaybackPipelineBase::UpdateVideoProcessorProperties(const wchar_t* name)
             GetParamDescription(nameStr.c_str(), description);
             if(description.m_Type == ParamVideoProcessor)
             {
-                m_pVideoProcessor->SetProperty(description.m_Name.c_str(), value);
+                m_pVideoConverter->SetProperty(description.m_Name.c_str(), value);
             }
         }
     }
@@ -1321,7 +1539,7 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
         pOutput->GetProperty(AMF_STREAM_EXTRA_DATA, &pInterface);
         pExtraData = amf::AMFBufferPtr(pInterface);
 
-        AMFSize frameSize;
+        AMFSize frameSize = {};
         pOutput->GetProperty(AMF_STREAM_VIDEO_FRAME_SIZE, &frameSize);
         m_iVideoWidth = frameSize.width;
         m_iVideoHeight = frameSize.height;
@@ -1362,6 +1580,10 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
     GetParam(PARAM_NAME_FULLSCREEN, bFullScreen);
     m_pVideoPresenter->SetFullScreen(bFullScreen);
 
+    bool bExclusiveFullScreen = false;
+    GetParam(PARAM_NAME_EXCLUSIVE_FULLSCREEN, bExclusiveFullScreen);
+    m_pVideoPresenter->SetExclusiveFullscreen(bExclusiveFullScreen);
+
     bool bEnablePIP = false;
     GetParam(PARAM_NAME_PIP, bEnablePIP);
     m_pVideoPresenter->SetEnablePIP(bEnablePIP);
@@ -1381,10 +1603,17 @@ AMF_RESULT  PlaybackPipelineBase::InitVideo(amf::AMFOutput* pOutput, amf::AMF_ME
     CHECK_AMF_ERROR_RETURN(res, L"m_pVideoPresenter->Init(" << m_iVideoWidth << m_iVideoHeight << ") failed");
 
     //---------------------------------------------------------------------------------------------
-    // Init Video Converter/Processor
+    // Init Video Converter/Processor/FRC
+    res = InitFRC(presenterEngine);
+    CHECK_AMF_ERROR_RETURN(res, L"InitFRC() failed");
+
     res = InitVideoProcessor();
     CHECK_AMF_ERROR_RETURN(res, L"InitVideoProcessor() failed");
 
+    res = InitColorConverter();
+    CHECK_AMF_ERROR_RETURN(res, L"InitColorConverter() failed");
+
+
     return AMF_OK;
 }
 
@@ -1413,9 +1642,9 @@ void PlaybackPipelineBase::OnEof()
             {
                 m_pVideoDecoder->ReInit(m_iVideoWidth, m_iVideoHeight);
             }
-            if(m_pVideoProcessor != NULL)
+            if(m_pVideoConverter != NULL)
             {
-                m_pVideoProcessor->ReInit(m_iVideoWidth, m_iVideoHeight);
+                m_pVideoConverter->ReInit(m_iVideoWidth, m_iVideoHeight);
             }
             if (m_pScaler != NULL)
             {
@@ -1425,6 +1654,10 @@ void PlaybackPipelineBase::OnEof()
             {
                 m_pHQScaler2->ReInit(m_iVideoWidth, m_iVideoHeight);
             }
+            if (m_pFRC != NULL)
+            {
+                m_pFRC->ReInit(m_iVideoWidth, m_iVideoHeight);
+            }
             if(m_pVideoPresenter != NULL)
             {
                 m_pVideoPresenter->Reset();
@@ -1457,7 +1690,7 @@ AMF_RESULT PlaybackPipelineBase::OnActivate(bool bActivated)
             GetParam(PARAM_NAME_PIP, bEnablePIP);
             m_pVideoPresenter->SetEnablePIP(bEnablePIP);
 
-            AMFPoint PIPFocusPos;
+            AMFPoint PIPFocusPos = {};
             GetParam(PARAM_NAME_PIP_FOCUS_X, PIPFocusPos.x);
             GetParam(PARAM_NAME_PIP_FOCUS_Y, PIPFocusPos.y);
             AMFFloatPoint2D fPIPFocusPos = { (amf_float)PIPFocusPos.x / (amf_float)m_iVideoWidth, (amf_float)PIPFocusPos.y / (amf_float)m_iVideoHeight };
@@ -1475,7 +1708,7 @@ AMF_RESULT PlaybackPipelineBase::OnActivate(bool bActivated)
 
 AMF_RESULT PlaybackPipelineBase::ReInit()
 {
-    //restart the playback with possibably different pipeline 
+    //restart the playback with possibably different pipeline
     if (GetState() == PipelineStateRunning)
     {
         Init();
@@ -1488,11 +1721,11 @@ AMF_RESULT PlaybackPipelineBase::ReInit()
             GetParam(PARAM_NAME_PIP, bEnablePIP);
             m_pVideoPresenter->SetEnablePIP(bEnablePIP);
 
-            amf_int iPIPZoomFactor;
+            amf_int iPIPZoomFactor = 0;
             GetParam(PARAM_NAME_PIP_ZOOM_FACTOR, iPIPZoomFactor);
             m_pVideoPresenter->SetPIPZoomFactor(iPIPZoomFactor);
 
-            AMFPoint PIPFocusPos;
+            AMFPoint PIPFocusPos = {};
             GetParam(PARAM_NAME_PIP_FOCUS_X, PIPFocusPos.x);
             GetParam(PARAM_NAME_PIP_FOCUS_Y, PIPFocusPos.y);
             AMFFloatPoint2D fPIPFocusPos = { (amf_float)PIPFocusPos.x / (amf_float)m_iVideoWidth, (amf_float)PIPFocusPos.y / (amf_float)m_iVideoHeight };
