@@ -41,6 +41,7 @@
 #include "public/include/components/FFMPEGFileDemuxer.h"
 #include "public/include/components/FFMPEGVideoDecoder.h"
 #include "public/include/components/HQScaler.h"
+#include "public/include/components/FRC.h"
 #include "BitStreamParser.h"
 #include "VideoPresenter.h"
 #include "AudioPresenter.h"
@@ -77,6 +78,20 @@ public:
     static const wchar_t* PARAM_NAME_PIP_FOCUS_X;
     static const wchar_t* PARAM_NAME_PIP_FOCUS_Y;
     static const wchar_t* PARAM_NAME_SIDE_BY_SIDE;
+    static const wchar_t* PARAM_NAME_HQ_SCALER_RGB;
+    static const wchar_t* PARAM_NAME_HQ_SCALER_RATIO;
+    static const wchar_t* PARAM_NAME_ENABLE_AUDIO;
+    static const wchar_t* PARAM_NAME_HQSCALER_SHARPNESS;
+    static const wchar_t* PARAM_NAME_FRAME_RATE;
+    static const wchar_t* PARAM_NAME_FRC_ENGINE;
+    static const wchar_t* PARAM_NAME_FRC_INFO;
+    static const wchar_t* PARAM_NAME_FRC_MODE;
+    static const wchar_t* PARAM_NAME_FRC_ENABLE_FALLBACK;
+    static const wchar_t* PARAM_NAME_FRC_INDICATOR;
+    static const wchar_t* PARAM_NAME_EXCLUSIVE_FULLSCREEN;
+    static const wchar_t* PARAM_NAME_FRC_PROFILE;
+    static const wchar_t* PARAM_NAME_FRC_MV_SEARCH_MODE;
+    static const wchar_t* PARAM_NAME_FRC_RGB;
 
     virtual AMF_RESULT Play();
     virtual AMF_RESULT Pause();
@@ -117,6 +132,10 @@ protected:
     virtual AMF_RESULT  InitVideo(amf::AMFOutput* pOutput, amf::AMF_MEMORY_TYPE presenterEngine);
 	virtual AMF_RESULT  InitVideoPipeline(amf_uint32 iVideoStreamIndex, PipelineElementPtr pVideoSourceStream);
 	virtual AMF_RESULT  InitAudioPipeline(amf_uint32 iAudioStreamIndex, PipelineElementPtr pAudioSourceStream);
+    virtual AMF_RESULT  ReInit();
+    virtual AMF_RESULT  InitFRC(amf::AMF_MEMORY_TYPE type);
+    virtual AMF_RESULT  InitColorConverter();
+    virtual AMF_RESULT  ConnectScaler();
 
     amf::AMFContextPtr      m_pContext;
 
@@ -127,9 +146,10 @@ protected:
     BitStreamParserPtr      m_pVideoStreamParser;
 
     amf::AMFComponentPtr    m_pVideoDecoder;
-    amf::AMFComponentPtr    m_pVideoProcessor;
+    amf::AMFComponentPtr    m_pVideoConverter;
     amf::AMFComponentPtr    m_pHQScaler2;
     amf::AMFComponentPtr    m_pScaler;
+    amf::AMFComponentPtr    m_pFRC;
     VideoPresenterPtr       m_pVideoPresenter;
     SplitterPtr             m_pSplitter;
     CombinerPtr             m_pCombiner;
